@@ -208,7 +208,7 @@ int parseDhcpMsgRecv() /* this routine parses dhcp message received */
 	    if (p + 2 + p[1] >= end)
 	      goto swend; /* corrupt packet */
 
-	    if (len = decodeSearch(p+2, p[1], NULL))
+	    if ((len = decodeSearch(p+2, p[1], NULL)))
 	      {
 		if ( DhcpOptions.val[*p] )
 		  free(DhcpOptions.val[*p]);
@@ -691,7 +691,8 @@ void (*buildUdpIpMsg)(unsigned);
 {
   struct sockaddr addr;
   struct timeval begin, current, diff;
-  int i,len,o,timeout=0;
+  int i,len,timeout=0;
+  socklen_t addrLength;
   char foobuf[512];
   const struct udphdr *udpRecv;
   int j=DHCP_INITIAL_RTO/2;
@@ -724,9 +725,9 @@ void (*buildUdpIpMsg)(unsigned);
 	  struct ip ipRecv_local;
   	  char *tmp_ip;
 	  memset(&UdpIpMsgRecv,0,sizeof(udpipMessage));
-      	  o=sizeof(struct sockaddr);
+      	  addrLength=sizeof(struct sockaddr);
       	  len=recvfrom(dhcpSocket,&UdpIpMsgRecv,sizeof(udpipMessage),0,
-		     (struct sockaddr *)&addr,&o);
+		     (struct sockaddr *)&addr,&addrLength);
 	  if ( len == -1 )
     	    {
       	      syslog(LOG_ERR,"recvfrom: %m\n");
@@ -829,6 +830,7 @@ void (*buildUdpIpMsg)(unsigned);
 /*****************************************************************************/
 void *dhcpStart()
 {
+  int flags;
   int o = 1;
   unsigned i=0;
   struct ifreq	ifr;
@@ -841,7 +843,9 @@ void *dhcpStart()
 #else
   dhcpSocket = socket(AF_PACKET,SOCK_PACKET,htons(ETH_P_ALL));
 #endif
-  if ( dhcpSocket == -1 )
+  if (dhcpSocket == -1 ||
+      (flags = fcntl(dhcpSocket, F_GETFL, 0)) == -1 ||
+      fcntl(dhcpSocket, F_SETFL, flags | O_NONBLOCK) == -1)
     {
       syslog(LOG_ERR,"dhcpStart: socket: %m\n");
       exit(1);
@@ -942,7 +946,7 @@ void *dhcpStart()
   ip_id=i&0xffff;
 
   udpFooSocket = socket(AF_INET,SOCK_DGRAM,0);
-  if ( udpFooSocket == -1 )
+  if (udpFooSocket == -1)
     {
       syslog(LOG_ERR,"dhcpStart: socket: %m\n");
       exit(1);
@@ -1123,7 +1127,10 @@ void (*buildDhcpMsg)(unsigned);
 /*****************************************************************************/
 void *dhcpBound()
 {
-  int i;
+  int i, maxfd;
+  fd_set rset;
+  char foobuf[512];
+
   if ( sigsetjmp(env,0xffff) ) return &dhcpRenew;
   i=ReqSentTime+ntohl(*(unsigned int *)DhcpOptions.val[dhcpT1value])-time(NULL);
   if ( i > 0 )
@@ -1131,8 +1138,41 @@ void *dhcpBound()
   else
     return &dhcpRenew;
 
-  sleep(ntohl(*(u_int *)DhcpOptions.val[dhcpT1value]));
-  return &dhcpRenew;
+  /* This used to be a sleep, but sleeping for hours/days with sockets open
+     on ports where broadcasts happen means wasting kernel buffer memory 
+     on hundreds of useless packets. 
+
+     This code waits in the same way, but it wakes up and reads any unexpected
+     packets to free the buffers. This waits "forever", and is 
+     interrupted by a siglongjump on receipt of sigalarm
+  */
+
+  while(1) 
+    {
+      FD_ZERO(&rset);
+      maxfd = dhcpSocket;
+      FD_SET(dhcpSocket, &rset);
+      if (udpFooSocket  != -1)
+	{
+	  if (udpFooSocket > maxfd)
+	    maxfd = udpFooSocket;
+	  FD_SET(udpFooSocket, &rset);
+	}
+      
+      if (select(maxfd+1, &rset, NULL, NULL, NULL) == -1)
+	{
+	  if (errno != EINTR)
+	    return &dhcpRenew;
+	}
+      else
+	{
+	  if (udpFooSocket != -1 && FD_ISSET(udpFooSocket, &rset))
+	    while (recvfrom(udpFooSocket,(void *)foobuf,sizeof(foobuf),0,NULL,NULL) != -1 );
+	  
+	  if (FD_ISSET(dhcpSocket, &rset))
+	    while (recvfrom(dhcpSocket,(void *)foobuf,sizeof(foobuf),0,NULL,NULL) != -1 );
+	}
+    }
 }
 /*****************************************************************************/
 void *dhcpRenew()
