@@ -44,6 +44,7 @@
 #include "client.h"
 
 extern	int			dhcpSocket;
+extern  int                     udpFooSocket;
 extern	int			prev_ip_addr;
 extern	int			Window;
 extern  int			SetDHCPDefaultRoutes;
@@ -112,6 +113,11 @@ char *prm;
 #endif
     {
       char *argc[5],exec_on_change[128];
+      
+      close(dhcpSocket);
+      if (udpFooSocket != -1)
+	close(udpFooSocket);
+
       if ( Cfilename )
 	snprintf(exec_on_change,sizeof(exec_on_change),Cfilename);
       else
@@ -388,12 +394,16 @@ int dhcpConfig()
       f=fopen(resolv_file, "w");
       if ( f )
 	{
-	  fprintf(f, "# Generated by dhcpcd for interface %s\n", IfName);
 	  int i;
+	  fprintf(f, "# Generated by dhcpcd for interface %s\n", IfName);
 	  if ( DhcpOptions.len[dnsSearchPath] )
 	    fprintf(f,"search %s\n", (char *)DhcpOptions.val[dnsSearchPath]);
-	  else if ( DhcpOptions.len[domainName] )
-	    fprintf(f,"domain %s\n",(char *)DhcpOptions.val[domainName]);
+	  else if ( DhcpOptions.len[domainName] ) {
+		if ( strchr( (char *)DhcpOptions.val[domainName], ' ') )
+		    fprintf(f,"search %s\n",(char *)DhcpOptions.val[domainName]);
+		else
+		    fprintf(f,"domain %s\n",(char *)DhcpOptions.val[domainName]);
+	  }
 	  
 	  for (i=0;i<DhcpOptions.len[dns];i+=4)
 	    fprintf(f,"nameserver %u.%u.%u.%u\n",
@@ -422,9 +432,9 @@ int dhcpConfig()
       f=fopen(nis_file, "w");
       if ( f )
 	{
-	  fprintf(f, "# Generated by dhcpcd for interface %s\n", IfName);
 	  int i;
 	  char *prefix=NULL;
+	  fprintf(f, "# Generated by dhcpcd for interface %s\n", IfName);
 	  if ( DhcpOptions.len[nisDomainName] ) {
 	      if ( DhcpOptions.len[nisServers] ) {
 		prefix=(char *)malloc(DhcpOptions.len[nisDomainName] + 15);
@@ -457,8 +467,10 @@ int dhcpConfig()
       f=fopen(ntp_file, "w");
       if ( f )
  	{
-	  fprintf(f, "# Generated by dhcpcd for interface %s\n", IfName);
  	  int net, mask;
+	  int i;
+	  char addr[4*3+3*1+1];
+	  fprintf(f, "# Generated by dhcpcd for interface %s\n", IfName);
  	  memcpy(&mask,DhcpOptions.val[subnetMask],4);
  	  net = DhcpIface.ciaddr & mask;
 
@@ -466,17 +478,15 @@ int dhcpConfig()
  	  fprintf(f,"restrict default noquery notrust nomodify\n");
  	  fprintf(f,"restrict 127.0.0.1\n");
   
-	      int i;
-	      char addr[4*3+3*1+1];
-	      for (i=0;i<DhcpOptions.len[ntpServers];i+=4)
-		{
-		  snprintf(addr,sizeof(addr),"%u.%u.%u.%u",
-		  ((unsigned char *)DhcpOptions.val[ntpServers])[i],
-		  ((unsigned char *)DhcpOptions.val[ntpServers])[i+1],
-		  ((unsigned char *)DhcpOptions.val[ntpServers])[i+2],
-		  ((unsigned char *)DhcpOptions.val[ntpServers])[i+3]);
-		  fprintf(f,"restrict %s nomodify notrap noquery\nserver %s\n",addr,addr);
-		}
+	  for (i=0;i<DhcpOptions.len[ntpServers];i+=4)
+	  {
+	    snprintf(addr,sizeof(addr),"%u.%u.%u.%u",
+	    ((unsigned char *)DhcpOptions.val[ntpServers])[i],
+	    ((unsigned char *)DhcpOptions.val[ntpServers])[i+1],
+	    ((unsigned char *)DhcpOptions.val[ntpServers])[i+2],
+	    ((unsigned char *)DhcpOptions.val[ntpServers])[i+3]);
+	    fprintf(f,"restrict %s nomodify notrap noquery\nserver %s\n",addr,addr);
+	  }
  	  
 	  fprintf(f, "driftfile /etc/ntp.drift\n");
  	  fprintf(f, "logfile /var/log/ntp.log\n");
